"""Airzone library common code."""

from __future__ import annotations

from enum import Enum, IntEnum
from typing import Any


class AirzoneStages(IntEnum):
    """Airzone stages."""

    UNKNOWN = -1

    Off = 0
    Air = 1
    Radiant = 2
    Combined = 3

    @classmethod
    def _missing_(cls, value: Any) -> AirzoneStages:
        return cls.UNKNOWN

    def exists(self) -> bool:
        """Return if Airzone Stage exits."""
        return self.value != self.Off

    def to_list(self) -> list[AirzoneStages]:
        """Convert AirzoneStages value to list."""
        if self.value == self.Combined:
            return [
                AirzoneStages.Off,
                AirzoneStages.Air,
                AirzoneStages.Radiant,
                AirzoneStages.Combined,
            ]
        if self.value in (self.Air, self.Radiant):
            return [AirzoneStages.Off, self]
        return []


class EcoAdapt(str, Enum):
    """Airzone Eco-Adapt."""

    OFF = "off"
    MANUAL = "manual"
    A = "a"
    A_PLUS = "a_p"
    A_PLUS_PLUS = "a_pp"

    @classmethod
    def _missing_(cls, value: Any) -> EcoAdapt:
        return cls.OFF


class GrilleAngle(IntEnum):
    """Airzone grille angles."""

    DEG_90 = 0
    DEG_50 = 1
    DEG_45 = 2
    DEG_40 = 3

    @classmethod
    def _missing_(cls, value: Any) -> GrilleAngle:
        return cls.DEG_90


class OperationAction(IntEnum):
    """Airzone operation actions."""

    COOLING = 1
    DRYING = 2
    FAN = 3
    HEATING = 4
    IDLE = 5
    OFF = 6


class OperationMode(IntEnum):
    """Airzone operation modes."""

    UNKNOWN = -1

    STOP = 1
    COOLING = 2
    HEATING = 3
    FAN = 4
    DRY = 5
    AUTO = 7

    @classmethod
    def _missing_(cls, value: Any) -> OperationMode:
        return cls.UNKNOWN


class HotWaterOperation(IntEnum):
    """Airzone Hot Water operations."""

    UNKNOWN = -1

    Off = 0
    On = 1
    Powerful = 2

    @classmethod
    def _missing_(cls, value: Any) -> HotWaterOperation:
        return cls.UNKNOWN


class SleepTimeout(IntEnum):
    """Airzone sleep timeouts."""

    SLEEP_OFF = 0
    SLEEP_30 = 30
    SLEEP_60 = 60
    SLEEP_90 = 90

    @classmethod
    def _missing_(cls, value: Any) -> SleepTimeout:
        return cls.SLEEP_OFF


class SystemType(IntEnum):
    """Airzone System Types."""

    UNKNOWN = -1

    C6 = 1
    AQUAGLASS = 2
    DZK = 3
    RADIANT = 4
    C3 = 5
    ZBS = 6
    ZS6 = 7

    @classmethod
    def _missing_(cls, value: Any) -> SystemType:
        return cls.UNKNOWN

    def __str__(self) -> str:
        """Convert ThermostatType value to string."""
        models: dict[int, str] = {
            self.UNKNOWN: "Unknown",
            self.C6: "C6",
            self.AQUAGLASS: "AQUAGLASS",
            self.DZK: "DZK",
            self.RADIANT: "Radiant",
            self.C3: "C3",
            self.ZBS: "ZBS",
            self.ZS6: "ZS6",
        }
        return models[self.value]


class TemperatureUnit(IntEnum):
    """Airzone temperature units."""

    CELSIUS = 0
    FAHRENHEIT = 1


class ThermostatType(IntEnum):
    """Airzone Thermostat Types."""

    UNKNOWN = -1

    Blueface = 1
    BluefaceZero = 2
    Lite = 3
    Think = 4

    @classmethod
    def _missing_(cls, value: Any) -> ThermostatType:
        return cls.UNKNOWN

    def __str__(self) -> str:
        """Convert ThermostatType value to string."""
        models: dict[int, str] = {
            self.UNKNOWN: "Unknown",
            self.Blueface: "Blueface",
            self.BluefaceZero: "Blueface Zero",
            self.Lite: "Lite",
            self.Think: "Think",
        }
        return models[self.value]

    def exists_radio(self) -> bool:
        """Return if a radio version of the Thermostat exists."""
        models: dict[int, bool] = {
            self.UNKNOWN: False,
            self.Blueface: False,
            self.BluefaceZero: False,
            self.Lite: True,
            self.Think: True,
        }
        return models[self.value]


class WebServerInterface(IntEnum):
    """Airzone WebServer Interface type."""

    UNKNOWN = -1

    ETHERNET = 1
    WIFI = 2

    @classmethod
    def _missing_(cls, value: Any) -> WebServerInterface:
        return cls.UNKNOWN


class WebServerType(IntEnum):
    """Airzone WebServer Types."""

    UNKNOWN = -1

    AIRZONE = 1
    AIDOO = 2

    @classmethod
    def _missing_(cls, value: Any) -> WebServerType:
        return cls.UNKNOWN

    def __str__(self) -> str:
        """Convert WebServerType value to string."""
        models: dict[int, str] = {
            self.UNKNOWN: "Unknown",
            self.AIRZONE: "Airzone WebServer",
            self.AIDOO: "Aidoo WebServer",
        }
        return models[self.value]


def get_system_zone_id(system_id: int, zone_id: int) -> str:
    """Combine system and zone IDs."""
    return f"{system_id}:{zone_id}"



"""Airzone Local API Device."""

from __future__ import annotations

from typing import Any

from .common import EcoAdapt, OperationMode, SystemType
from .const import (
    API_ECO_ADAPT,
    API_ERRORS,
    API_MANUFACTURER,
    API_MC_CONNECTED,
    API_MODE,
    API_POWER,
    API_SYSTEM_FIRMWARE,
    API_SYSTEM_TYPE,
    AZD_AVAILABLE,
    AZD_CLAMP_METER,
    AZD_ECO_ADAPT,
    AZD_ENERGY,
    AZD_ERRORS,
    AZD_FIRMWARE,
    AZD_FULL_NAME,
    AZD_ID,
    AZD_MANUFACTURER,
    AZD_MASTER_SYSTEM_ZONE,
    AZD_MASTER_ZONE,
    AZD_MODE,
    AZD_MODEL,
    AZD_MODES,
    AZD_PROBLEMS,
    ERROR_SYSTEM,
)
from .zone import Zone


class System:
    """Airzone System."""

    def __init__(self, system_id: int, zone_data: dict[str, Any]):
        """System init."""
        self.available: bool = True
        self.clamp_meter: bool | None = None
        self.eco_adapt: EcoAdapt | None = None
        self.energy: int | None = None
        self.errors: list[str] = []
        self.id: int = system_id
        self.firmware: str | None = None
        self.manufacturer: str | None = None
        self.master_system_zone: str | None = None
        self.master_zone: int | None = None
        self.mode: OperationMode | None = None
        self.modes: list[OperationMode] = []
        self.type: SystemType | None = None
        self.zones: dict[int, Zone] = {}

        self.update_zone_data(zone_data)

    def update_zone_data(self, zone_data: dict[str, Any]) -> None:
        """Update System data."""
        self.available = True

        errors: list[dict[str, str]] = zone_data.get(API_ERRORS, [])
        for error in errors:
            for key, val in error.items():
                self.add_error(val, key)

    def data(self) -> dict[str, Any]:
        """Return Airzone system data."""
        data: dict[str, Any] = {}

        data[AZD_AVAILABLE] = self.get_available()

        clamp_meter = self.get_clamp_meter()
        if clamp_meter is not None:
            data[AZD_CLAMP_METER] = clamp_meter

            if clamp_meter:
                energy = self.get_energy()
                if energy is not None:
                    data[AZD_ENERGY] = energy

        eco_adapt = self.get_eco_adapt()
        if eco_adapt is not None:
            data[AZD_ECO_ADAPT] = eco_adapt

        errors = self.get_errors()
        if len(errors) > 0:
            data[AZD_ERRORS] = errors

        firmware = self.get_firmware()
        if firmware is not None:
            data[AZD_FIRMWARE] = firmware

        full_name = self.get_full_name()
        if full_name is not None:
            data[AZD_FULL_NAME] = full_name

        data[AZD_ID] = self.get_id()

        manufacturer = self.get_manufacturer()
        if manufacturer is not None:
            data[AZD_MANUFACTURER] = manufacturer

        master_system_zone = self.get_master_system_zone()
        if master_system_zone is not None:
            data[AZD_MASTER_SYSTEM_ZONE] = master_system_zone

        master_zone = self.get_master_zone()
        if master_zone is not None:
            data[AZD_MASTER_ZONE] = master_zone

        mode = self.get_mode()
        if mode is not None:
            data[AZD_MODE] = mode

        model = self.get_model()
        if self.type is not None:
            data[AZD_MODEL] = model

        modes = self.get_modes()
        if modes is not None:
            data[AZD_MODES] = modes

        data[AZD_PROBLEMS] = self.get_problems()

        return data

    def add_error(self, error: str, error_id: str | None = None) -> None:
        """Add system error."""
        if error_id is not None:
            if error_id.casefold() == ERROR_SYSTEM and error not in self.errors:
                self.errors += [error]
        else:
            if error not in self.errors:
                self.errors += [error]

    def add_zone(self, zone: Zone) -> None:
        """Add zone to system."""
        zone_id = zone.get_id()
        if zone_id not in self.zones:
            self.zones[zone_id] = zone

    def get_available(self) -> bool:
        """Return availability."""
        return self.available

    def get_clamp_meter(self) -> bool | None:
        """Return system clamp meter connection."""
        return self.clamp_meter

    def get_eco_adapt(self) -> EcoAdapt | None:
        """Return system Eco Adapt."""
        return self.eco_adapt

    def get_energy(self) -> int | None:
        """Return system energy consumption."""
        return self.energy

    def get_errors(self) -> list[str]:
        """Return system errors."""
        return self.errors

    def get_id(self) -> int:
        """Return system ID."""
        return self.id

    def get_firmware(self) -> str | None:
        """Return system firmware."""
        if self.firmware and "." not in self.firmware and len(self.firmware) > 2:
            return f"{self.firmware[0:1]}.{self.firmware[1:]}"
        return self.firmware

    def get_full_name(self) -> str:
        """Return full name."""
        return f"Airzone [{self.get_id()}] System"

    def get_manufacturer(self) -> str | None:
        """Return system manufacturer."""
        return self.manufacturer

    def get_master_system_zone(self) -> str | None:
        """Return master system zone ID."""
        return self.master_system_zone

    def get_master_zone(self) -> int | None:
        """Return master zone ID."""
        return self.master_zone

    def get_model(self) -> str | None:
        """Return system model."""
        if self.type:
            return str(self.type)
        return None

    def get_mode(self) -> OperationMode | None:
        """Return system mode."""
        return self.mode

    def get_modes(self) -> list[OperationMode]:
        """Return system modes."""
        modes = self.modes

        if len(modes) == 0 and self.mode is not None:
            modes = [self.mode]

        if OperationMode.STOP not in modes:
            modes += [OperationMode.STOP]

        return modes

    def get_problems(self) -> bool:
        """Return system problems."""
        return bool(self.errors)

    def set_available(self, available: bool) -> None:
        """Set availability."""
        self.available = available

    def set_eco_adapt(self, eco_adapt: EcoAdapt | None) -> None:
        """Set system Eco Adapt."""
        self.eco_adapt = eco_adapt

    def set_master_system_zone(self, master_system_zone: str) -> None:
        """Set master system zone ID."""
        self.master_system_zone = master_system_zone

    def set_master_zone(self, master_zone: int) -> None:
        """Set master zone ID."""
        self.master_zone = master_zone

    def set_mode(self, mode: OperationMode | None) -> None:
        """Set system mode."""
        self.mode = mode

    def set_modes(self, modes: list[OperationMode]) -> None:
        """Set system modes."""
        self.modes = modes

    def set_param(self, key: str, value: Any) -> None:
        """Update parameters by key and value."""
        if key == API_ECO_ADAPT:
            self.eco_adapt = EcoAdapt(value)
        elif key == API_MODE:
            self.mode = OperationMode(value)

        for zone in self.zones.values():
            zone.set_param(key, value)

    def update_data(self, data: dict[str, Any]) -> None:
        """Update system parameters by dict."""

        self.available = True

        if API_MC_CONNECTED in data:
            self.clamp_meter = bool(data[API_MC_CONNECTED])

        if API_ERRORS in data:
            errors: list[dict[str, str]] = data[API_ERRORS]
            for error in errors:
                for val in error.values():
                    self.add_error(val)

        if API_MANUFACTURER in data:
            self.manufacturer = str(data[API_MANUFACTURER])

        if API_POWER in data:
            self.energy = int(data[API_POWER])

        if API_SYSTEM_FIRMWARE in data:
            self.firmware = str(data[API_SYSTEM_FIRMWARE])

        if API_SYSTEM_TYPE in data:
            self.type = SystemType(data[API_SYSTEM_TYPE])



"""Airzone Local API Thermostat."""

from __future__ import annotations

from typing import Any

from .common import ThermostatType
from .const import (
    API_THERMOS_FIRMWARE,
    API_THERMOS_RADIO,
    API_THERMOS_TYPE,
    THERMOSTAT_RADIO,
    THERMOSTAT_WIRED,
)


class Thermostat:
    """Airzone Thermostat."""

    def __init__(self, data: dict[str, Any]):
        """Thermostat init."""
        self.firmware: str | None = None
        self.radio: bool | None = None
        self.type: ThermostatType | None = None

        if API_THERMOS_FIRMWARE in data:
            self.firmware = str(data[API_THERMOS_FIRMWARE])
        if API_THERMOS_RADIO in data:
            self.radio = bool(data[API_THERMOS_RADIO])
        if API_THERMOS_TYPE in data:
            self.type = ThermostatType(data[API_THERMOS_TYPE])

    def get_firmware(self) -> str | None:
        """Return Airzone Thermostat firmware."""
        if self.firmware and "." not in self.firmware and len(self.firmware) > 2:
            return f"{self.firmware[0:1]}.{self.firmware[1:]}"
        return self.firmware

    def get_model(self) -> str | None:
        """Return Airzone Thermostat model."""
        if self.type:
            name = str(self.type)
            if self.type.exists_radio():
                sfx = f" ({THERMOSTAT_RADIO if self.radio else THERMOSTAT_WIRED})"
            else:
                sfx = ""
            return f"{name}{sfx}"
        return None

    def get_radio(self) -> bool | None:
        """Return Airzone Thermostat radio."""
        return self.radio



"""Airzone Local API."""

from __future__ import annotations

from dataclasses import dataclass
from enum import IntEnum
import json
from json import JSONDecodeError
import logging
from typing import Any, cast

from aiohttp import ClientConnectorError, ClientSession
from aiohttp.client_reqrep import ClientResponse

from .common import OperationMode, get_system_zone_id
from .const import (
    API_ACS_MAX_TEMP,
    API_ACS_MIN_TEMP,
    API_ACS_ON,
    API_ACS_SET_POINT,
    API_ACS_TEMP,
    API_DATA,
    API_DEMO,
    API_DHW_PARAMS,
    API_ERROR_HOT_WATER_NOT_CONNECTED,
    API_ERROR_METHOD_NOT_SUPPORTED,
    API_ERROR_REQUEST_MALFORMED,
    API_ERROR_SYSTEM_ID_NOT_AVAILABLE,
    API_ERROR_SYSTEM_ID_OUT_RANGE,
    API_ERROR_ZONE_ID_NOT_AVAILABLE,
    API_ERROR_ZONE_ID_NOT_PROVIDED,
    API_ERROR_ZONE_ID_OUT_RANGE,
    API_ERRORS,
    API_HVAC,
    API_INTEGRATION,
    API_MAC,
    API_NO_FEEDBACK_PARAMS,
    API_SYSTEM_ID,
    API_SYSTEM_PARAMS,
    API_SYSTEMS,
    API_V1,
    API_VERSION,
    API_WEBSERVER,
    API_ZONE_ID,
    API_ZONE_PARAMS,
    AZD_HOT_WATER,
    AZD_NEW_SYSTEMS,
    AZD_NEW_ZONES,
    AZD_SYSTEMS,
    AZD_SYSTEMS_NUM,
    AZD_VERSION,
    AZD_WEBSERVER,
    AZD_ZONES,
    AZD_ZONES_NUM,
    DEFAULT_PORT,
    DEFAULT_SYSTEM_ID,
    HTTP_CALL_TIMEOUT,
    RAW_DEMO,
    RAW_DHW,
    RAW_HVAC,
    RAW_INTEGRATION,
    RAW_SYSTEMS,
    RAW_VERSION,
    RAW_WEBSERVER,
)
from .exceptions import (
    APIError,
    HotWaterNotAvailable,
    InvalidHost,
    InvalidMethod,
    InvalidParam,
    InvalidSystem,
    InvalidZone,
    RequestMalformed,
    SystemNotAvailable,
    SystemOutOfRange,
    ZoneNotAvailable,
    ZoneNotProvided,
    ZoneOutOfRange,
)
from .hotwater import HotWater
from .system import System
from .webserver import WebServer
from .zone import Zone

_LOGGER = logging.getLogger(__name__)


class ApiFeature(IntEnum):
    """Supported features of the Airzone Local API."""

    HVAC = 0
    SYSTEMS = 1
    WEBSERVER = 2
    HOT_WATER = 4


@dataclass
class ConnectionOptions:
    """Airzone Local API options for connection."""

    host: str
    port: int = DEFAULT_PORT
    system_id: int = DEFAULT_SYSTEM_ID


class AirzoneLocalApi:
    """Airzone Local API device representation."""

    def __init__(
        self,
        aiohttp_session: ClientSession,
        options: ConnectionOptions,
    ):
        """Device init."""
        self._api_raw_data: dict[str, Any] = {}
        self._first_update: bool = True
        self._new_systems: list[str] = []
        self._new_zones: list[str] = []
        self.aiohttp_session = aiohttp_session
        self.api_features: int = ApiFeature.HVAC
        self.api_features_checked = False
        self.hotwater: HotWater | None = None
        self.options = options
        self.systems: dict[int, System] = {}
        self.version: str | None = None
        self.webserver: WebServer | None = None
        self.zones: dict[str, Zone] = {}

    @staticmethod
    def handle_errors(errors: list[dict[str, str]]) -> None:
        """Handle API errors."""
        for error in errors:
            for key, val in error.items():
                if val == API_ERROR_HOT_WATER_NOT_CONNECTED:
                    raise HotWaterNotAvailable(f"{key}: {val}")
                if val == API_ERROR_METHOD_NOT_SUPPORTED:
                    raise InvalidMethod(f"{key}: {val}")
                if val == API_ERROR_REQUEST_MALFORMED:
                    raise RequestMalformed(f"{key}: {val}")
                if val == API_ERROR_SYSTEM_ID_NOT_AVAILABLE:
                    raise SystemNotAvailable(f"{key}: {val}")
                if val == API_ERROR_SYSTEM_ID_OUT_RANGE:
                    raise SystemOutOfRange(f"{key}: {val}")
                if val == API_ERROR_ZONE_ID_OUT_RANGE:
                    raise ZoneOutOfRange(f"{key}: {val}")
                if val == API_ERROR_ZONE_ID_NOT_AVAILABLE:
                    raise ZoneNotAvailable(f"{key}: {val}")
                if val == API_ERROR_ZONE_ID_NOT_PROVIDED:
                    raise ZoneNotProvided(f"{key}: {val}")
                raise APIError(f"{key}: {val}")

    async def http_request(
        self, method: str, path: str, data: Any | None = None
    ) -> dict[str, Any]:
        """Device HTTP request."""
        _LOGGER.debug("aiohttp request: /%s (params=%s)", path, data)

        try:
            resp: ClientResponse = await self.aiohttp_session.request(
                method,
                f"http://{self.options.host}:{self.options.port}/{path}",
                data=json.dumps(data),
                timeout=HTTP_CALL_TIMEOUT,
            )
        except ClientConnectorError as err:
            raise InvalidHost(err) from err

        try:
            resp_json = await resp.json(content_type=None)
        except JSONDecodeError as err:
            raise InvalidHost(err) from err
        except UnicodeDecodeError:
            # Workaround bogus encoding
            try:
                resp_json = await resp.json(encoding="iso-8859-1", content_type=None)
            except JSONDecodeError as err:
                raise InvalidHost(err) from err

        _LOGGER.debug("aiohttp response: %s", resp_json)
        if resp.status != 200:
            if API_ERRORS in resp_json:
                self.handle_errors(resp_json[API_ERRORS])
            raise APIError(f"HTTP status: {resp.status}")

        return cast(dict[str, Any], resp_json)

    def update_dhw(self, data: dict[str, Any]) -> None:
        """Gather Domestic Hot Water data."""
        dhw = data.get(API_DATA, {})
        if self.hotwater is not None:
            self.hotwater.update_data(dhw)
        else:
            self.hotwater = HotWater(dhw)

    def update_systems(self, data: dict[str, Any]) -> None:
        """Gather Systems data."""
        api_systems = data.get(API_SYSTEMS)
        if api_systems is None:
            raise APIError(f"update_systems: {API_SYSTEMS} not in API response")
        for api_system in api_systems:
            system = self.get_system(api_system[API_SYSTEM_ID])
            if system:
                system.update_data(api_system)

    def update_webserver(self, data: dict[str, Any]) -> None:
        """Gather WebServer data."""
        self.webserver = WebServer(data)

    def check_dhw(self, dhw: dict[str, Any]) -> bool:
        """Check Airzone Domestic Hot Water validity."""
        return all(
            [
                dhw.get(API_ACS_MAX_TEMP, 0) != 0,
                dhw.get(API_ACS_MIN_TEMP, 0) != 0,
                API_ACS_ON in dhw,
                dhw.get(API_ACS_SET_POINT, 0) != 0,
                dhw.get(API_ACS_TEMP, 0) != 0,
                dhw.get(API_SYSTEM_ID, 0) == 0,
            ]
        )

    async def check_features(self, update: bool) -> None:
        """Check Airzone API features."""
        try:
            self.webserver = None
            webserver = await self.get_webserver()
            if API_MAC in webserver:
                self.api_features |= ApiFeature.WEBSERVER
                self.update_webserver(webserver)
        except InvalidMethod:
            pass

        try:
            systems = await self.get_hvac_systems()
            if API_SYSTEMS in systems:
                self.api_features |= ApiFeature.SYSTEMS
                if update:
                    self.update_systems(systems)
        except (SystemOutOfRange, ZoneNotProvided):
            pass

        try:
            dhw = await self.get_dhw()
            if self.check_dhw(dhw.get(API_DATA, {})):
                self.api_features |= ApiFeature.HOT_WATER
                if update:
                    self.update_dhw(dhw)
        except (HotWaterNotAvailable, ZoneNotProvided):
            pass

        try:
            version = await self.get_version()
            if API_VERSION in version:
                self.version = version[API_VERSION]
        except InvalidMethod:
            pass

        self.api_features_checked = True

    async def update_features(self) -> None:
        """Gather Airzone features data."""
        if not self.api_features_checked:
            await self.check_features(True)
        else:
            if self.api_features & ApiFeature.HOT_WATER:
                self.update_dhw(await self.get_dhw())

            if self.api_features & ApiFeature.SYSTEMS:
                self.update_systems(await self.get_hvac_systems())

            if self.api_features & ApiFeature.WEBSERVER:
                self.update_webserver(await self.get_webserver())

    async def validate(self) -> str | None:
        """Validate Airzone API."""
        self._first_update = True

        await self.check_features(False)

        response = await self.get_hvac()
        if self.options.system_id == DEFAULT_SYSTEM_ID:
            if API_SYSTEMS not in response:
                raise InvalidHost(f"validate: {API_SYSTEMS} not in API response")
        elif API_DATA not in response:
            raise InvalidHost(f"validate: {API_DATA} not in API response")

        if self.webserver:
            return self.webserver.get_mac()

        return None

    async def update(self) -> None:
        """Gather Airzone data."""

        for system in self.systems.values():
            system.set_available(False)
        for zone in self.zones.values():
            zone.set_available(False)

        hvac = await self.get_hvac()
        if self.options.system_id == DEFAULT_SYSTEM_ID:
            hvac_systems = hvac.get(API_SYSTEMS)
            if hvac_systems is None:
                raise APIError(f"update: {API_SYSTEMS} not in API response")
            for system_data in hvac_systems:
                self.parse_system_zones(system_data)
        else:
            self.parse_system_zones(hvac)

        await self.update_features()

        self._first_update = False

    def parse_system_zones(self, system_data: dict[str, Any]) -> None:
        """Parse all zones from system data."""
        self._new_systems = []
        self._new_zones = []

        system_zones: list[dict[str, Any]] = system_data.get(API_DATA, [])
        for zone_data in system_zones:
            system_id = int(zone_data.get(API_SYSTEM_ID, 0))
            if system_id > 0:
                if system_id not in self.systems:
                    self.systems[system_id] = System(system_id, zone_data)
                    self._new_systems += [str(system_id)]
                else:
                    self.systems[system_id].update_zone_data(zone_data)

                zone_id = int(zone_data.get(API_ZONE_ID, 0))
                if zone_id > 0:
                    system_zone_id = get_system_zone_id(system_id, zone_id)

                    if system_zone_id not in self.zones:
                        _zone = Zone(system_id, zone_id, zone_data)
                        self.zones[system_zone_id] = _zone
                        self.systems[system_id].add_zone(_zone)
                        self._new_zones += [system_zone_id]
                    else:
                        self.zones[system_zone_id].update_data(zone_data)

                    self.update_system_from_zone(
                        self.systems[system_id], self.zones[system_zone_id]
                    )

        self.update_zones_from_master_zone()

        if self._first_update:
            self._new_systems = []
            self._new_zones = []
        else:
            if len(self._new_systems) > 0:
                _LOGGER.debug("New systems detected: %s", self._new_systems)
            if len(self._new_zones) > 0:
                _LOGGER.debug("New zones detected: %s", self._new_zones)

    def update_system_from_zone(self, system: System, zone: Zone) -> None:
        """Update system data from zone."""
        if zone.get_master():
            if (eco_adapt := zone.get_eco_adapt()) is not None:
                system.set_eco_adapt(eco_adapt)

            system.set_master_system_zone(zone.get_system_zone_id())
            system.set_master_zone(zone.get_id())

            if (mode := zone.get_mode()) is not None:
                system.set_mode(mode)

            system.set_modes(zone.get_modes())
        else:
            if system.get_eco_adapt() is None:
                system.set_eco_adapt(zone.get_eco_adapt())
            if system.get_mode() is None:
                system.set_mode(zone.get_mode())
            if len(system.get_modes()) == 0:
                system.set_modes(zone.get_modes())

    def update_zones_from_master_zone(self) -> None:
        """Update slave zones data with their master zone."""
        for zone in self.zones.values():
            system_id = zone.get_system_id()

            if system_id is not None and not zone.get_master():
                modes: list[OperationMode] = []

                master_id = zone.get_master_zone()
                if master_id is None:
                    system = self.get_system(system_id)
                    modes = system.get_modes()
                else:
                    master_zone = self.get_zone(system_id, master_id)
                    modes = master_zone.get_modes()

                if len(modes) > 0:
                    zone.set_modes(modes)

    async def get_demo(self) -> dict[str, Any]:
        """Return Airzone demo."""
        res = await self.http_request(
            "POST",
            f"{API_V1}/{API_DEMO}",
        )
        self._api_raw_data[RAW_DEMO] = res
        return res

    async def get_dhw(self, params: dict[str, Any] | None = None) -> dict[str, Any]:
        """Return Airzone DHW (Domestic Hot Water)."""
        if not params:
            params = {
                API_SYSTEM_ID: 0,
            }
        res = await self.http_request(
            "POST",
            f"{API_V1}/{API_HVAC}",
            params,
        )
        self._api_raw_data[RAW_DHW] = res
        return res

    async def get_hvac_systems(
        self, params: dict[str, Any] | None = None
    ) -> dict[str, Any]:
        """Return Airzone HVAC systems."""
        if not params:
            params = {
                API_SYSTEM_ID: 127,
            }
        res = await self.http_request(
            "POST",
            f"{API_V1}/{API_HVAC}",
            params,
        )
        self._api_raw_data[RAW_SYSTEMS] = res
        return res

    async def get_hvac(self, params: dict[str, Any] | None = None) -> dict[str, Any]:
        """Return Airzone HVAC zones."""
        if not params:
            params = {
                API_SYSTEM_ID: self.options.system_id,
                API_ZONE_ID: 0,
            }
        res = await self.http_request(
            "POST",
            f"{API_V1}/{API_HVAC}",
            params,
        )
        self._api_raw_data[RAW_HVAC] = res
        return res

    async def get_integration(self) -> dict[str, Any]:
        """Return Airzone integration."""
        res = await self.http_request(
            "POST",
            f"{API_V1}/{API_INTEGRATION}",
        )
        self._api_raw_data[RAW_INTEGRATION] = res
        return res

    async def get_version(self) -> dict[str, Any]:
        """Return Airzone Local API version."""
        res = await self.http_request(
            "POST",
            f"{API_V1}/{API_VERSION}",
        )
        self._api_raw_data[RAW_VERSION] = res
        return res

    async def get_webserver(self) -> dict[str, Any]:
        """Return Airzone WebServer."""
        res = await self.http_request(
            "POST",
            f"{API_V1}/{API_WEBSERVER}",
        )
        self._api_raw_data[RAW_WEBSERVER] = res
        return res

    async def put_hvac(self, params: dict[str, Any]) -> dict[str, Any]:
        """Perform a PUT request to update HVAC parameters."""
        return await self.http_request(
            "PUT",
            f"{API_V1}/{API_HVAC}",
            params,
        )

    async def set_dhw_parameters(self, params: dict[str, Any]) -> dict[str, Any]:
        """Set Airzone Hot Water parameters and handle response."""
        res = await self.put_hvac(params)

        if API_DATA not in res:
            if API_ERRORS in res:
                self.handle_errors(res[API_ERRORS])
            raise APIError(f"set_dhw: {API_DATA} not in API response")

        if self.hotwater is not None:
            data: dict[str, Any] = res.get(API_DATA, {})

            for key, value in data.items():
                if key in API_DHW_PARAMS:
                    self.hotwater.set_param(key, value)

        return res

    async def set_hvac_parameters(self, params: dict[str, Any]) -> dict[str, Any]:
        """Set Airzone HVAC parameters and handle response."""
        res = await self.put_hvac(params)

        if API_DATA not in res:
            if API_ERRORS in res:
                self.handle_errors(res[API_ERRORS])
            raise APIError(f"set_hvac: {API_DATA} not in API response")

        data: dict[str, Any] = res[API_DATA][0]

        for param in API_NO_FEEDBACK_PARAMS:
            value = params.get(param)
            if value is not None and param not in data:
                _LOGGER.debug("set_hvac: forcing %s=%s", param, value)
                data[param] = value

        for key, value in params.items():
            if (
                key in [API_SYSTEM_ID, API_ZONE_ID]
                and value != 0
                and data.get(key) != value
            ):
                if key == API_SYSTEM_ID:
                    raise InvalidSystem(
                        f"set_hvac: System mismatch: {data.get(key)} vs {value}"
                    )
                if key == API_ZONE_ID:
                    raise InvalidZone(
                        f"set_hvac: Zone mismatch: {data.get(key)} vs {value}"
                    )

            if key not in data:
                raise InvalidParam(f"set_hvac: param not in data: {key}={value}")

        system = self.get_system(data[API_SYSTEM_ID])
        zone = self.get_zone(data[API_SYSTEM_ID], data[API_ZONE_ID])
        for key, value in data.items():
            if key in API_SYSTEM_PARAMS:
                system.set_param(key, value)
            elif key in API_ZONE_PARAMS:
                zone.set_param(key, value)

        return res

    def new_systems(self) -> list[str]:
        """Return new systems detected in last update."""
        return self._new_systems

    def new_zones(self) -> list[str]:
        """Return new zones detected in last update."""
        return self._new_zones

    def raw_data(self) -> dict[str, Any]:
        """Return raw Airzone API data."""
        return self._api_raw_data

    def data(self) -> dict[str, Any]:
        """Return Airzone device data."""
        data: dict[str, Any] = {}

        if self.hotwater is not None:
            data[AZD_HOT_WATER] = self.hotwater.data()

        data[AZD_NEW_SYSTEMS] = self.new_systems()

        data[AZD_SYSTEMS_NUM] = self.num_systems()
        if len(self.systems) > 0:
            systems: dict[int, Any] = {}
            for system_id, system in self.systems.items():
                systems[system_id] = system.data()
            data[AZD_SYSTEMS] = systems

        if self.webserver is not None:
            data[AZD_WEBSERVER] = self.webserver.data()

        data[AZD_NEW_ZONES] = self.new_zones()

        data[AZD_ZONES_NUM] = self.num_zones()
        if len(self.zones) > 0:
            zones: dict[str, Any] = {}
            for system_zone_id, zone in self.zones.items():
                zones[system_zone_id] = zone.data()
            data[AZD_ZONES] = zones

        if self.version is not None:
            data[AZD_VERSION] = self.version

        return data

    def get_system(self, system_id: int) -> System:
        """Return Airzone system."""
        if system_id not in self.systems:
            raise InvalidSystem(f"System {system_id} not present")
        return self.systems[system_id]

    def get_zone(self, system_id: int, zone_id: int) -> Zone:
        """Return Airzone zone."""
        system_zone_id = get_system_zone_id(system_id, zone_id)
        if system_zone_id not in self.zones:
            raise InvalidZone(f"Zone {system_zone_id} not present")
        return self.zones[system_zone_id]

    def num_systems(self) -> int:
        """Return number of systems."""
        return len(self.systems)

    def num_zones(self) -> int:
        """Return number of zones."""
        return len(self.zones)



"""Airzone library constants."""

from typing import Final

API_ACS_MAX_TEMP: Final[str] = "acs_maxtemp"
API_ACS_MIN_TEMP: Final[str] = "acs_mintemp"
API_ACS_ON: Final[str] = "acs_power"
API_ACS_POWER_MODE: Final[str] = "acs_powerful"
API_ACS_SET_POINT: Final[str] = "acs_setpoint"
API_ACS_TEMP: Final[str] = "acs_temp"
API_AIR_DEMAND: Final[str] = "air_demand"
API_ANTI_FREEZE: Final[str] = "antifreeze"
API_COLD_ANGLE: Final[str] = "coldangle"
API_COLD_DEMAND: Final[str] = "cold_demand"
API_COLD_STAGE: Final[str] = "coldStage"
API_COLD_STAGES: Final[str] = "coldStages"
API_COOL_MAX_TEMP: Final[str] = "coolmaxtemp"
API_COOL_MIN_TEMP: Final[str] = "coolmintemp"
API_COOL_SET_POINT: Final[str] = "coolsetpoint"
API_DATA: Final[str] = "data"
API_DEMO: Final[str] = "demo"
API_DRIVER: Final[str] = "driver"
API_DOUBLE_SET_POINT: Final[str] = "double_sp"
API_ECO_ADAPT: Final[str] = "eco_adapt"
API_ERROR: Final[str] = "error"
API_ERRORS: Final[str] = "errors"
API_FLOOR_DEMAND: Final[str] = "floor_demand"
API_HEAT_ANGLE: Final[str] = "heatangle"
API_HEAT_DEMAND: Final[str] = "heat_demand"
API_HEAT_MAX_TEMP: Final[str] = "heatmaxtemp"
API_HEAT_MIN_TEMP: Final[str] = "heatmintemp"
API_HEAT_SET_POINT: Final[str] = "heatsetpoint"
API_HEAT_STAGE: Final[str] = "heatStage"
API_HEAT_STAGES: Final[str] = "heatStages"
API_HUMIDITY: Final[str] = "humidity"
API_HVAC: Final[str] = "hvac"
API_INTEGRATION: Final[str] = "integration"
API_INTERFACE: Final[str] = "interface"
API_MAC: Final[str] = "mac"
API_MANUFACTURER: Final[str] = "manufacturer"
API_MASTER_ZONE_ID: Final[str] = "master_zoneID"
API_MAX_TEMP: Final[str] = "maxTemp"
API_MC_CONNECTED: Final[str] = "mc_connected"
API_MIN_TEMP: Final[str] = "minTemp"
API_MODE: Final[str] = "mode"
API_MODES: Final[str] = "modes"
API_NAME: Final[str] = "name"
API_ON: Final[str] = "on"
API_POWER: Final[str] = "power"
API_ROOM_TEMP: Final[str] = "roomTemp"
API_SET_POINT: Final[str] = "setpoint"
API_SLEEP: Final[str] = "sleep"
API_SPEED: Final[str] = "speed"
API_SPEEDS: Final[str] = "speeds"
API_SYSTEM_FIRMWARE: Final[str] = "system_firmware"
API_SYSTEM_ID: Final[str] = "systemID"
API_SYSTEM_TYPE: Final[str] = "system_type"
API_SYSTEMS: Final[str] = "systems"
API_TEMP_STEP: Final[str] = "temp_step"
API_THERMOS_FIRMWARE: Final[str] = "thermos_firmware"
API_THERMOS_RADIO: Final[str] = "thermos_radio"
API_THERMOS_TYPE: Final[str] = "thermos_type"
API_UNITS: Final[str] = "units"
API_V1: Final[str] = "api/v1"
API_VERSION: Final[str] = "version"
API_WEBSERVER: Final[str] = "webserver"
API_WIFI: Final[str] = "wifi"
API_WIFI_CHANNEL: Final[str] = "wifi_channel"
API_WIFI_QUALITY: Final[str] = "wifi_quality"
API_WIFI_RSSI: Final[str] = "wifi_rssi"
API_WS_AIDOO: Final[str] = "ws_aidoo"
API_WS_AZ: Final[str] = "ws_az"
API_WS_FIRMWARE: Final[str] = "ws_firmware"
API_WS_TYPE: Final[str] = "ws_type"
API_ZONE_ID: Final[str] = "zoneID"

API_ERROR_HOT_WATER_NOT_CONNECTED: Final[str] = "acs not connected"
API_ERROR_LOW_BATTERY: Final[str] = "Low battery"
API_ERROR_METHOD_NOT_SUPPORTED: Final[str] = "Method not provided or not supported"
API_ERROR_REQUEST_MALFORMED: Final[str] = "request malformed"
API_ERROR_SYSTEM_ID_NOT_AVAILABLE: Final[str] = "systemid not avaiable"
API_ERROR_SYSTEM_ID_NOT_PROVIDED: Final[str] = "systemid not provided"
API_ERROR_SYSTEM_ID_OUT_RANGE: Final[str] = "systemid out of range"
API_ERROR_ZONE_ID_NOT_AVAILABLE: Final[str] = "zoneid not avaiable"
API_ERROR_ZONE_ID_NOT_PROVIDED: Final[str] = "zoneid not provided"
API_ERROR_ZONE_ID_OUT_RANGE: Final[str] = "zoneid out of range"

API_DHW_PARAMS: Final[list[str]] = [
    API_ACS_ON,
    API_ACS_POWER_MODE,
    API_ACS_SET_POINT,
]
API_DOUBLE_SET_POINT_PARAMS: Final[set[str]] = {
    API_COOL_MAX_TEMP,
    API_COOL_MIN_TEMP,
    API_COOL_SET_POINT,
    API_HEAT_MAX_TEMP,
    API_HEAT_MIN_TEMP,
    API_HEAT_SET_POINT,
}
API_NO_FEEDBACK_PARAMS: Final[list[str]] = [
    API_MODE,
]
API_SYSTEM_PARAMS: Final[list[str]] = [
    API_MODE,
    API_SPEED,
]
API_ZONE_PARAMS: Final[list[str]] = [
    API_COOL_SET_POINT,
    API_COLD_ANGLE,
    API_COLD_STAGE,
    API_HEAT_ANGLE,
    API_HEAT_SET_POINT,
    API_HEAT_STAGE,
    API_NAME,
    API_ON,
    API_SET_POINT,
    API_SLEEP,
]

AZD_ABS_TEMP_MAX: Final[str] = "absolute-temp-max"
AZD_ABS_TEMP_MIN: Final[str] = "absolute-temp-min"
AZD_ACTION: Final[str] = "action"
AZD_AIR_DEMAND: Final[str] = "air-demand"
AZD_ANTI_FREEZE: Final[str] = "anti-freeze"
AZD_AVAILABLE: Final[str] = "available"
AZD_BATTERY_LOW: Final[str] = "battery-low"
AZD_CLAMP_METER: Final[str] = "clamp-meter"
AZD_COLD_ANGLE: Final[str] = "cold-angle"
AZD_COLD_DEMAND: Final[str] = "cold-demand"
AZD_COLD_STAGE: Final[str] = "cold-stage"
AZD_COLD_STAGES: Final[str] = "cold-stages"
AZD_COOL_TEMP_MAX: Final[str] = "cool-temp-max"
AZD_COOL_TEMP_MIN: Final[str] = "cool-temp-min"
AZD_COOL_TEMP_SET: Final[str] = "cool-temp-set"
AZD_DEMAND: Final[str] = "demand"
AZD_DOUBLE_SET_POINT: Final[str] = "double-set-point"
AZD_ECO_ADAPT: Final[str] = "eco-adapt"
AZD_ENERGY: Final[str] = "energy"
AZD_ERRORS: Final[str] = "errors"
AZD_FIRMWARE: Final[str] = "firmware"
AZD_FULL_NAME: Final[str] = "full-name"
AZD_FLOOR_DEMAND: Final[str] = "floor-demand"
AZD_HEAT_ANGLE: Final[str] = "heat-angle"
AZD_HEAT_DEMAND: Final[str] = "heat-demand"
AZD_HEAT_TEMP_MAX: Final[str] = "heat-temp-max"
AZD_HEAT_TEMP_MIN: Final[str] = "heat-temp-min"
AZD_HEAT_TEMP_SET: Final[str] = "heat-temp-set"
AZD_HEAT_STAGE: Final[str] = "heat-stage"
AZD_HEAT_STAGES: Final[str] = "heat-stages"
AZD_HOT_WATER: Final[str] = "hot-water"
AZD_HUMIDITY: Final[str] = "humidity"
AZD_ID: Final[str] = "id"
AZD_INTERFACE: Final[str] = "interface"
AZD_MAC: Final[str] = "mac"
AZD_MANUFACTURER: Final[str] = "manufacturer"
AZD_MASTER: Final[str] = "master"
AZD_MASTER_SYSTEM_ZONE: Final[str] = "master-system-zone"
AZD_MASTER_ZONE: Final[str] = "master-zone"
AZD_MODE: Final[str] = "mode"
AZD_MODEL: Final[str] = "model"
AZD_MODES: Final[str] = "modes"
AZD_NAME: Final[str] = "name"
AZD_NEW_SYSTEMS: Final[str] = "new-systems"
AZD_NEW_ZONES: Final[str] = "new-zones"
AZD_OPERATION: Final[str] = "operation"
AZD_OPERATIONS: Final[str] = "operations"
AZD_ON: Final[str] = "on"
AZD_POWER_MODE: Final[str] = "power-mode"
AZD_PROBLEMS: Final[str] = "problems"
AZD_SLEEP: Final[str] = "sleep"
AZD_SPEED: Final[str] = "speed"
AZD_SPEEDS: Final[str] = "speeds"
AZD_SYSTEM: Final[str] = "system"
AZD_SYSTEMS: Final[str] = "systems"
AZD_SYSTEMS_NUM: Final[str] = "num-systems"
AZD_TEMP: Final[str] = "temp"
AZD_TEMP_MAX: Final[str] = "temp-max"
AZD_TEMP_MIN: Final[str] = "temp-min"
AZD_TEMP_SET: Final[str] = "temp-set"
AZD_TEMP_STEP: Final[str] = "temp-step"
AZD_TEMP_UNIT: Final[str] = "temp-unit"
AZD_THERMOSTAT_FW: Final[str] = "thermostat-fw"
AZD_THERMOSTAT_MODEL: Final[str] = "thermostat-model"
AZD_THERMOSTAT_RADIO: Final[str] = "thermostat-radio"
AZD_VERSION: Final[str] = "version"
AZD_WEBSERVER: Final[str] = "webserver"
AZD_WIFI_CHANNEL: Final[str] = "wifi-channel"
AZD_WIFI_QUALITY: Final[str] = "wifi-quality"
AZD_WIFI_RSSI: Final[str] = "wifi-rssi"
AZD_ZONES: Final[str] = "zones"
AZD_ZONES_NUM: Final[str] = "num-zones"

API_BUG_MAX_TEMP_FAH: Final[int] = 140

DEFAULT_PORT: Final[int] = 3000
DEFAULT_SYSTEM_ID: Final[int] = 0
DEFAULT_TEMP_MAX_CELSIUS: Final[float] = 30.0
DEFAULT_TEMP_MAX_FAHRENHEIT: Final[float] = 86.0
DEFAULT_TEMP_STEP_CELSIUS: Final[float] = 0.5
DEFAULT_TEMP_STEP_FAHRENHEIT: Final[float] = 1.0

ERROR_SYSTEM: Final[str] = "system"
ERROR_ZONE: Final[str] = "zone"

HTTP_CALL_TIMEOUT: Final[int] = 10

RAW_DEMO: Final[str] = "demo"
RAW_DHW: Final[str] = "dhw"
RAW_HVAC: Final[str] = "hvac"
RAW_INTEGRATION: Final[str] = "integration"
RAW_SYSTEMS: Final[str] = "systems"
RAW_VERSION: Final[str] = "version"
RAW_WEBSERVER: Final[str] = "webserver"

THERMOSTAT_RADIO: Final[str] = "Radio"
THERMOSTAT_WIRED: Final[str] = "Wired"



"""Airzone Local API Domestic Hot Water."""

from __future__ import annotations

from typing import Any

from .common import HotWaterOperation, TemperatureUnit
from .const import (
    API_ACS_MAX_TEMP,
    API_ACS_MIN_TEMP,
    API_ACS_ON,
    API_ACS_POWER_MODE,
    API_ACS_SET_POINT,
    API_ACS_TEMP,
    API_UNITS,
    AZD_NAME,
    AZD_ON,
    AZD_OPERATION,
    AZD_OPERATIONS,
    AZD_POWER_MODE,
    AZD_TEMP,
    AZD_TEMP_MAX,
    AZD_TEMP_MIN,
    AZD_TEMP_SET,
    AZD_TEMP_UNIT,
)


class HotWater:
    """Airzone Domestic Hot Water."""

    def __init__(self, data: dict[str, Any]):
        """Hot Water init."""
        self.name: str = "Airzone DHW"
        self.on: bool
        self.temp: int
        self.temp_max: int
        self.temp_min: int
        self.temp_set: int
        self.temp_unit: TemperatureUnit = TemperatureUnit.CELSIUS
        self.power_mode: bool | None = None
        self.update_data(data)

    def update_data(self, data: dict[str, Any]) -> None:
        """Update Hot Water data."""
        self.on = bool(data[API_ACS_ON])
        self.temp = int(data[API_ACS_TEMP])
        self.temp_max = int(data[API_ACS_MAX_TEMP])
        self.temp_min = int(data[API_ACS_MIN_TEMP])
        self.temp_set = int(data[API_ACS_SET_POINT])

        if API_ACS_POWER_MODE in data:
            self.power_mode = bool(data[API_ACS_POWER_MODE])

        if API_UNITS in data:
            self.temp_unit = TemperatureUnit(data[API_UNITS])

    def data(self) -> dict[str, Any]:
        """Return Airzone Hot Water data."""
        data: dict[str, Any] = {
            AZD_NAME: self.get_name(),
            AZD_ON: self.get_on(),
            AZD_OPERATION: self.get_operation(),
            AZD_OPERATIONS: self.get_operations(),
            AZD_TEMP: self.get_temp(),
            AZD_TEMP_MAX: self.get_temp_max(),
            AZD_TEMP_MIN: self.get_temp_min(),
            AZD_TEMP_SET: self.get_temp_set(),
            AZD_TEMP_UNIT: self.get_temp_unit(),
        }

        power_mode = self.get_power_mode()
        if power_mode is not None:
            data[AZD_POWER_MODE] = power_mode

        return data

    def get_name(self) -> str | None:
        """Return Hot Water name."""
        return self.name

    def get_on(self) -> bool:
        """Return Hot Water on/off."""
        return self.on

    def get_operation(self) -> HotWaterOperation:
        """Return Hot Water current operation."""
        if self.get_on():
            if self.get_power_mode():
                return HotWaterOperation.Powerful
            return HotWaterOperation.On
        return HotWaterOperation.Off

    def get_operations(self) -> list[HotWaterOperation]:
        """Return Hot Water operation list."""
        operations = [
            HotWaterOperation.Off,
            HotWaterOperation.On,
        ]
        if self.get_power_mode() is not None:
            operations += [HotWaterOperation.Powerful]
        return operations

    def get_power_mode(self) -> bool | None:
        """Return Hot Water power mode on/off."""
        if self.power_mode is not None:
            return self.power_mode
        return None

    def get_temp(self) -> int:
        """Return Hot Water temperature."""
        return self.temp

    def get_temp_max(self) -> int:
        """Return Hot Water max temperature."""
        return self.temp_max

    def get_temp_min(self) -> int:
        """Return Hot Water min temperature."""
        return self.temp_min

    def get_temp_set(self) -> int:
        """Return Hot Water set temperature."""
        return self.temp_set

    def get_temp_unit(self) -> TemperatureUnit:
        """Return Hot Water temperature unit."""
        return self.temp_unit

    def set_param(self, key: str, value: Any) -> None:
        """Update Hot Water parameter by key and value."""
        if key == API_ACS_ON:
            self.on = bool(value)
        elif key == API_ACS_POWER_MODE:
            if self.power_mode is not None:
                self.power_mode = bool(value)
        elif key == API_ACS_SET_POINT:
            self.temp_set = int(value)



"""Airzone Local API Device."""

from __future__ import annotations

from typing import Any

from .common import (
    AirzoneStages,
    EcoAdapt,
    GrilleAngle,
    OperationAction,
    OperationMode,
    SleepTimeout,
    TemperatureUnit,
    get_system_zone_id,
)
from .const import (
    API_AIR_DEMAND,
    API_ANTI_FREEZE,
    API_BUG_MAX_TEMP_FAH,
    API_COLD_ANGLE,
    API_COLD_DEMAND,
    API_COLD_STAGE,
    API_COLD_STAGES,
    API_COOL_MAX_TEMP,
    API_COOL_MIN_TEMP,
    API_COOL_SET_POINT,
    API_DOUBLE_SET_POINT,
    API_DOUBLE_SET_POINT_PARAMS,
    API_ECO_ADAPT,
    API_ERROR_LOW_BATTERY,
    API_ERRORS,
    API_FLOOR_DEMAND,
    API_HEAT_ANGLE,
    API_HEAT_DEMAND,
    API_HEAT_MAX_TEMP,
    API_HEAT_MIN_TEMP,
    API_HEAT_SET_POINT,
    API_HEAT_STAGE,
    API_HEAT_STAGES,
    API_HUMIDITY,
    API_MASTER_ZONE_ID,
    API_MAX_TEMP,
    API_MIN_TEMP,
    API_MODE,
    API_MODES,
    API_NAME,
    API_ON,
    API_ROOM_TEMP,
    API_SET_POINT,
    API_SLEEP,
    API_SPEED,
    API_SPEEDS,
    API_TEMP_STEP,
    API_UNITS,
    AZD_ABS_TEMP_MAX,
    AZD_ABS_TEMP_MIN,
    AZD_ACTION,
    AZD_AIR_DEMAND,
    AZD_ANTI_FREEZE,
    AZD_AVAILABLE,
    AZD_BATTERY_LOW,
    AZD_COLD_ANGLE,
    AZD_COLD_DEMAND,
    AZD_COLD_STAGE,
    AZD_COLD_STAGES,
    AZD_COOL_TEMP_MAX,
    AZD_COOL_TEMP_MIN,
    AZD_COOL_TEMP_SET,
    AZD_DEMAND,
    AZD_DOUBLE_SET_POINT,
    AZD_ECO_ADAPT,
    AZD_ERRORS,
    AZD_FLOOR_DEMAND,
    AZD_FULL_NAME,
    AZD_HEAT_ANGLE,
    AZD_HEAT_DEMAND,
    AZD_HEAT_STAGE,
    AZD_HEAT_STAGES,
    AZD_HEAT_TEMP_MAX,
    AZD_HEAT_TEMP_MIN,
    AZD_HEAT_TEMP_SET,
    AZD_HUMIDITY,
    AZD_ID,
    AZD_MASTER,
    AZD_MASTER_ZONE,
    AZD_MODE,
    AZD_MODES,
    AZD_NAME,
    AZD_ON,
    AZD_PROBLEMS,
    AZD_SLEEP,
    AZD_SPEED,
    AZD_SPEEDS,
    AZD_SYSTEM,
    AZD_TEMP,
    AZD_TEMP_MAX,
    AZD_TEMP_MIN,
    AZD_TEMP_SET,
    AZD_TEMP_STEP,
    AZD_TEMP_UNIT,
    AZD_THERMOSTAT_FW,
    AZD_THERMOSTAT_MODEL,
    AZD_THERMOSTAT_RADIO,
    DEFAULT_TEMP_MAX_CELSIUS,
    DEFAULT_TEMP_MAX_FAHRENHEIT,
    DEFAULT_TEMP_STEP_CELSIUS,
    DEFAULT_TEMP_STEP_FAHRENHEIT,
    ERROR_ZONE,
)
from .thermostat import Thermostat


class Zone:
    """Airzone Zone."""

    def __init__(self, system_id: int, zone_id: int, zone_data: dict[str, Any]):
        """Zone init."""
        self.air_demand: bool | None = None
        self.anti_freeze: bool | None = None
        self.available: bool = True
        self.cold_angle: GrilleAngle | None = None
        self.cold_demand: bool | None = None
        self.cold_stage: AirzoneStages | None = None
        self.cold_stages: list[AirzoneStages] = []
        self.cool_temp_max: float | None = None
        self.cool_temp_min: float | None = None
        self.cool_temp_set: float | None = None
        self.double_set_point: bool | None = None
        self.eco_adapt: EcoAdapt | None = None
        self.errors: list[str] = []
        self.floor_demand: bool | None = None
        self.heat_angle: GrilleAngle | None = None
        self.heat_demand: bool | None = None
        self.heat_temp_max: float | None = None
        self.heat_temp_min: float | None = None
        self.heat_temp_set: float | None = None
        self.heat_stage: AirzoneStages | None = None
        self.heat_stages: list[AirzoneStages] = []
        self.humidity: int | None = None
        self.id = zone_id
        self.master_zone: int | None = None
        self.mode: OperationMode
        self.modes: list[OperationMode] = []
        self.on: bool
        self.sleep: SleepTimeout | None = None
        self.speed: int | None = None
        self.speeds: list[int] = []
        self.system_id: int = system_id
        self.system_zone_id: str = get_system_zone_id(system_id, zone_id)
        self.temp_set: float | None = None

        self.name: str = f"Airzone {self.get_system_zone_id()}"

        self.update_data(zone_data)

    def update_data(self, zone_data: dict[str, Any]) -> None:
        """Update Zone data."""
        self.available = True

        self.double_set_point_params: bool = (
            zone_data.keys() >= API_DOUBLE_SET_POINT_PARAMS
        )
        self.master = bool(API_MODES in zone_data)
        self.on = bool(zone_data[API_ON])
        self.temp = float(zone_data[API_ROOM_TEMP])
        self.temp_max = float(zone_data[API_MAX_TEMP])
        self.temp_min = float(zone_data[API_MIN_TEMP])
        self.temp_step: float | None = None
        self.temp_unit = TemperatureUnit(zone_data[API_UNITS])
        self.thermostat = Thermostat(zone_data)

        if API_MASTER_ZONE_ID in zone_data:
            master_zone_id = int(zone_data[API_MASTER_ZONE_ID])
            if master_zone_id != self.id:
                self.master_zone = master_zone_id

        if API_AIR_DEMAND in zone_data:
            self.air_demand = bool(zone_data[API_AIR_DEMAND])
        if API_FLOOR_DEMAND in zone_data:
            self.floor_demand = bool(zone_data[API_FLOOR_DEMAND])

        if API_ANTI_FREEZE in zone_data:
            self.anti_freeze = bool(zone_data[API_ANTI_FREEZE])

        if API_COLD_DEMAND in zone_data:
            self.cold_demand = bool(zone_data[API_COLD_DEMAND])
        if API_HEAT_DEMAND in zone_data:
            self.heat_demand = bool(zone_data[API_HEAT_DEMAND])

        if API_DOUBLE_SET_POINT in zone_data:
            self.double_set_point = bool(zone_data[API_DOUBLE_SET_POINT])

        if API_ECO_ADAPT in zone_data:
            self.eco_adapt = EcoAdapt(zone_data[API_ECO_ADAPT])

        if API_HUMIDITY in zone_data:
            self.humidity = int(zone_data[API_HUMIDITY])

        if API_COLD_ANGLE in zone_data:
            self.cold_angle = GrilleAngle(zone_data[API_COLD_ANGLE])
        if API_HEAT_ANGLE in zone_data:
            self.heat_angle = GrilleAngle(zone_data[API_HEAT_ANGLE])

        if API_COLD_STAGE in zone_data:
            self.cold_stage = AirzoneStages(zone_data[API_COLD_STAGE])
        if API_COLD_STAGES in zone_data:
            cold_stages = AirzoneStages(zone_data[API_COLD_STAGES])
            self.cold_stages = cold_stages.to_list()
        elif self.cold_stage and self.cold_stage.exists():
            self.cold_stages = [self.cold_stage]

        if API_HEAT_STAGE in zone_data:
            self.heat_stage = AirzoneStages(zone_data[API_HEAT_STAGE])
        if API_HEAT_STAGES in zone_data:
            heat_stages = AirzoneStages(zone_data[API_HEAT_STAGES])
            self.heat_stages = heat_stages.to_list()
        elif self.heat_stage and self.heat_stage.exists():
            self.heat_stages = [self.heat_stage]

        if API_COOL_MAX_TEMP in zone_data:
            self.cool_temp_max = float(zone_data[API_COOL_MAX_TEMP])
        if API_COOL_MIN_TEMP in zone_data:
            self.cool_temp_min = float(zone_data[API_COOL_MIN_TEMP])
        if API_COOL_SET_POINT in zone_data:
            cool_temp_set = self.validate_temp_set(
                float(zone_data[API_COOL_SET_POINT]), self.cool_temp_max
            )
            if cool_temp_set is not None:
                self.cool_temp_set = cool_temp_set

        if API_HEAT_MAX_TEMP in zone_data:
            self.heat_temp_max = float(zone_data[API_HEAT_MAX_TEMP])
        if API_HEAT_MIN_TEMP in zone_data:
            self.heat_temp_min = float(zone_data[API_HEAT_MIN_TEMP])
        if API_HEAT_SET_POINT in zone_data:
            heat_temp_set = self.validate_temp_set(
                float(zone_data[API_HEAT_SET_POINT]), self.heat_temp_max
            )
            if heat_temp_set is not None:
                self.heat_temp_set = heat_temp_set

        if API_ERRORS in zone_data:
            errors: list[dict[str, str]] = zone_data[API_ERRORS]
            for error in errors:
                for key, val in error.items():
                    self.add_error(key, val)

        if API_MODE in zone_data:
            self.mode = OperationMode(zone_data[API_MODE])
        else:
            self.master = True
            self.mode = OperationMode.AUTO
            self.modes = [self.mode]

        if self.master:
            if API_MODES in zone_data:
                self.modes = []
                for mode in zone_data[API_MODES]:
                    self.modes += [OperationMode(mode)]

        if API_NAME in zone_data:
            self.name = str(zone_data[API_NAME])

        if API_SLEEP in zone_data:
            self.sleep = SleepTimeout(zone_data[API_SLEEP])

        if API_SPEED in zone_data:
            self.speed = int(zone_data[API_SPEED])
        if API_SPEEDS in zone_data:
            speeds = int(zone_data[API_SPEEDS])
            self.speeds = list(range(0, speeds + 1))

        if API_SET_POINT in zone_data:
            temp_set = self.validate_temp_set(
                float(zone_data[API_SET_POINT]), self.temp_max
            )
            if temp_set is not None:
                self.temp_set = temp_set
        if API_TEMP_STEP in zone_data:
            self.temp_step = float(zone_data[API_TEMP_STEP])
        else:
            if self.temp_unit == TemperatureUnit.FAHRENHEIT:
                self.temp_step = DEFAULT_TEMP_STEP_FAHRENHEIT
            else:
                self.temp_step = DEFAULT_TEMP_STEP_CELSIUS

    def data(self) -> dict[str, Any]:
        """Return Airzone zone data."""
        data = {
            AZD_ABS_TEMP_MAX: self.get_abs_temp_max(),
            AZD_ABS_TEMP_MIN: self.get_abs_temp_min(),
            AZD_ACTION: self.get_action(),
            AZD_AVAILABLE: self.get_available(),
            AZD_DEMAND: self.get_demand(),
            AZD_DOUBLE_SET_POINT: self.get_double_set_point(),
            AZD_ID: self.get_id(),
            AZD_MASTER: self.get_master(),
            AZD_MODE: self.get_mode(),
            AZD_NAME: self.get_name(),
            AZD_ON: self.get_on(),
            AZD_PROBLEMS: self.get_problems(),
            AZD_SYSTEM: self.get_system_id(),
            AZD_TEMP: self.get_temp(),
            AZD_TEMP_MAX: self.get_temp_max(),
            AZD_TEMP_MIN: self.get_temp_min(),
            AZD_TEMP_UNIT: self.get_temp_unit(),
        }

        air_demand = self.get_air_demand()
        if air_demand is not None:
            data[AZD_AIR_DEMAND] = air_demand

        floor_demand = self.get_floor_demand()
        if floor_demand is not None:
            data[AZD_FLOOR_DEMAND] = floor_demand

        anti_freeze = self.get_anti_freeze()
        if anti_freeze is not None:
            data[AZD_ANTI_FREEZE] = anti_freeze

        eco_adapt = self.get_eco_adapt()
        if eco_adapt is not None:
            data[AZD_ECO_ADAPT] = eco_adapt

        full_name = self.get_full_name()
        if full_name is not None:
            data[AZD_FULL_NAME] = full_name

        humidity = self.get_humidity()
        if humidity is not None:
            data[AZD_HUMIDITY] = humidity

        cool_temp_max = self.get_cool_temp_max()
        if cool_temp_max:
            data[AZD_COOL_TEMP_MAX] = cool_temp_max
        cool_temp_min = self.get_cool_temp_min()
        if cool_temp_min:
            data[AZD_COOL_TEMP_MIN] = cool_temp_min
        cool_temp_set = self.get_cool_temp_set()
        if cool_temp_set:
            data[AZD_COOL_TEMP_SET] = cool_temp_set
        heat_temp_max = self.get_heat_temp_max()
        if heat_temp_max:
            data[AZD_HEAT_TEMP_MAX] = heat_temp_max
        heat_temp_min = self.get_heat_temp_min()
        if heat_temp_min:
            data[AZD_HEAT_TEMP_MIN] = heat_temp_min
        heat_temp_set = self.get_heat_temp_set()
        if heat_temp_set:
            data[AZD_HEAT_TEMP_SET] = heat_temp_set

        cold_angle = self.get_cold_angle()
        if cold_angle is not None:
            data[AZD_COLD_ANGLE] = cold_angle
        heat_angle = self.get_heat_angle()
        if heat_angle is not None:
            data[AZD_HEAT_ANGLE] = heat_angle

        cold_demand = self.get_cold_demand()
        if cold_demand is not None:
            data[AZD_COLD_DEMAND] = cold_demand
        heat_demand = self.get_heat_demand()
        if heat_demand is not None:
            data[AZD_HEAT_DEMAND] = heat_demand

        cold_stage = self.get_cold_stage()
        if cold_stage is not None:
            data[AZD_COLD_STAGE] = cold_stage
        cold_stages = self.get_cold_stages()
        if cold_stages is not None:
            data[AZD_COLD_STAGES] = cold_stages

        heat_stage = self.get_heat_stage()
        if heat_stage is not None:
            data[AZD_HEAT_STAGE] = heat_stage
        heat_stages = self.get_heat_stages()
        if heat_stages is not None:
            data[AZD_HEAT_STAGES] = heat_stages

        sleep = self.get_sleep()
        if sleep is not None:
            data[AZD_SLEEP] = sleep

        speed = self.get_speed()
        if speed is not None:
            data[AZD_SPEED] = speed
        speeds = self.get_speeds()
        if speeds is not None:
            data[AZD_SPEEDS] = speeds

        errors = self.get_errors()
        if len(errors) > 0:
            data[AZD_ERRORS] = errors

        master_zone = self.get_master_zone()
        if master_zone is not None:
            data[AZD_MASTER_ZONE] = master_zone

        modes = self.get_modes()
        if modes is not None:
            data[AZD_MODES] = modes

        temp_set = self.get_temp_set()
        if temp_set is not None:
            data[AZD_TEMP_SET] = temp_set
        temp_step = self.get_temp_step()
        if temp_step is not None:
            data[AZD_TEMP_STEP] = temp_step

        thermostat_firmware = self.thermostat.get_firmware()
        if thermostat_firmware is not None:
            data[AZD_THERMOSTAT_FW] = thermostat_firmware
        thermostat_model = self.thermostat.get_model()
        if thermostat_model is not None:
            data[AZD_THERMOSTAT_MODEL] = thermostat_model
        thermostat_radio = self.thermostat.get_radio()
        if thermostat_radio is not None:
            data[AZD_THERMOSTAT_RADIO] = thermostat_radio

        battery_low = self.get_battery_low()
        if battery_low is not None:
            data[AZD_BATTERY_LOW] = battery_low

        return data

    def add_error(self, key: str, val: str) -> None:
        """Add zone error."""
        _key = key.casefold()
        if _key == ERROR_ZONE and val not in self.errors:
            self.errors += [val]

    def fix_max_temp(self, max_temp: float) -> float:
        """Fix possibly bugged max temperatures."""
        if (
            self.get_temp_unit() == TemperatureUnit.FAHRENHEIT
            and max_temp >= API_BUG_MAX_TEMP_FAH
        ):
            max_temp = max_temp / 10
        if max_temp == 0.0:
            if self.get_temp_unit() == TemperatureUnit.FAHRENHEIT:
                max_temp = DEFAULT_TEMP_MAX_FAHRENHEIT
            else:
                max_temp = DEFAULT_TEMP_MAX_CELSIUS
        return round(max_temp, 1)

    def get_abs_temp_max(self) -> float:
        """Return absolute max temp."""
        temps = [
            self.get_cool_temp_max(),
            self.get_heat_temp_max(),
            self.get_temp_max(),
        ]
        return max(list(temp for temp in temps if temp is not None))

    def get_abs_temp_min(self) -> float:
        """Return absolute min temp."""
        temps = [
            self.get_cool_temp_min(),
            self.get_heat_temp_min(),
            self.get_temp_min(),
        ]
        return min(list(temp for temp in temps if temp is not None))

    def get_action(self) -> OperationAction:
        """Return zone action."""

        if self.get_on():
            if self.get_demand():
                mode = self.get_mode()
                if mode == OperationMode.COOLING:
                    action = OperationAction.COOLING
                elif mode == OperationMode.HEATING:
                    action = OperationAction.HEATING
                elif mode == OperationMode.FAN:
                    action = OperationAction.FAN
                elif mode == OperationMode.DRY:
                    action = OperationAction.DRYING
                elif mode == OperationMode.AUTO:
                    action = self.get_auto_mode()
                else:
                    action = OperationAction.OFF
            else:
                action = OperationAction.IDLE
        else:
            action = OperationAction.OFF

        return action

    def get_air_demand(self) -> bool | None:
        """Return zone air demand."""
        if self.air_demand is not None and self.is_stage_supported(AirzoneStages.Air):
            return self.air_demand
        return None

    def get_anti_freeze(self) -> bool | None:
        """Return zone anti freeze."""
        return self.anti_freeze

    def get_auto_mode(self) -> OperationAction:
        """Return action from auto mode."""
        temp_sp = self.get_temp_set()
        temp_min = self.get_temp_min()
        temp_max = self.get_temp_max()
        cool_sp = self.get_cool_temp_set()
        cool_max = self.get_cool_temp_max()
        cool_min = self.get_cool_temp_min()
        heat_sp = self.get_heat_temp_set()
        heat_max = self.get_heat_temp_max()
        heat_min = self.get_heat_temp_min()

        if (
            cool_max is not None
            and cool_min is not None
            and heat_max is not None
            and heat_min is not None
        ):
            cool_match = cool_max == temp_max and cool_min == temp_min
            heat_match = heat_max == temp_max and heat_min == temp_min

            if cool_match and not heat_match:
                return OperationAction.COOLING
            if heat_match and not cool_match:
                return OperationAction.HEATING

        if cool_sp is not None and heat_sp is not None:
            cool_match = cool_sp == temp_sp
            heat_match = heat_sp == temp_sp

            if cool_match and not heat_match:
                return OperationAction.COOLING
            if heat_match and not cool_match:
                return OperationAction.HEATING

        return OperationAction.IDLE

    def get_available(self) -> bool:
        """Return availability."""
        return self.available

    def get_battery_low(self) -> bool | None:
        """Return battery status."""
        if self.thermostat.get_radio():
            return API_ERROR_LOW_BATTERY in self.errors
        return None

    def get_cold_angle(self) -> GrilleAngle | None:
        """Return zone cold angle."""
        return self.cold_angle

    def get_cold_demand(self) -> bool | None:
        """Return zone cold demand."""
        return self.cold_demand

    def get_cold_stage(self) -> AirzoneStages | None:
        """Return zone cold stage."""
        return self.cold_stage

    def get_cold_stages(self) -> list[AirzoneStages] | None:
        """Return zone cold stages."""
        if len(self.cold_stages) > 0:
            return self.cold_stages
        return None

    def get_cool_temp_max(self) -> float | None:
        """Return zone maximum cool temperature."""
        if self.cool_temp_max is not None:
            return self.fix_max_temp(self.cool_temp_max)
        return None

    def get_cool_temp_min(self) -> float | None:
        """Return zone minimum cool temperature."""
        if self.cool_temp_min is not None:
            return round(self.cool_temp_min, 1)
        return None

    def get_cool_temp_set(self) -> float | None:
        """Return zone set cool temperature."""
        if self.cool_temp_set:
            return round(self.cool_temp_set, 1)
        return None

    def get_demand(self) -> bool:
        """Return zone demand."""
        return bool(self.air_demand) or bool(self.floor_demand)

    def get_double_set_point(self) -> bool:
        """Return zone double set point."""
        if self.double_set_point_params:
            if self.double_set_point is not None:
                return self.double_set_point
            return OperationMode.AUTO in self.get_modes()
        return False

    def get_eco_adapt(self) -> EcoAdapt | None:
        """Return zone echo adapt."""
        return self.eco_adapt

    def get_errors(self) -> list[str]:
        """Return zone errors."""
        return self.errors

    def get_floor_demand(self) -> bool | None:
        """Return zone floor demand."""
        if self.floor_demand is not None and self.is_stage_supported(
            AirzoneStages.Radiant
        ):
            return self.floor_demand
        return None

    def get_full_name(self) -> str:
        """Return full name."""
        return f"Airzone [{self.get_system_zone_id()}] {self.get_name()}"

    def get_id(self) -> int:
        """Return zone ID."""
        return self.id

    def get_heat_angle(self) -> GrilleAngle | None:
        """Return zone heat angle."""
        return self.heat_angle

    def get_heat_demand(self) -> bool | None:
        """Return zone heat demand."""
        return self.heat_demand

    def get_heat_stage(self) -> AirzoneStages | None:
        """Return zone heat stage."""
        return self.heat_stage

    def get_heat_stages(self) -> list[AirzoneStages] | None:
        """Return zone heat stages."""
        if len(self.heat_stages) > 0:
            return self.heat_stages
        return None

    def get_heat_temp_max(self) -> float | None:
        """Return zone maximum heat temperature."""
        if self.heat_temp_max is not None:
            return self.fix_max_temp(self.heat_temp_max)
        return None

    def get_heat_temp_min(self) -> float | None:
        """Return zone minimum heat temperature."""
        if self.heat_temp_min is not None:
            return round(self.heat_temp_min, 1)
        return None

    def get_heat_temp_set(self) -> float | None:
        """Return zone set heat temperature."""
        if self.heat_temp_set:
            return round(self.heat_temp_set, 1)
        return None

    def get_humidity(self) -> int | None:
        """Return zone humidity."""
        if self.humidity is not None and self.humidity != 0:
            return self.humidity
        return None

    def get_master(self) -> bool:
        """Return zone master/slave."""
        return self.master

    def get_master_zone(self) -> int | None:
        """Return corresponding master zone."""
        return self.master_zone

    def get_mode(self) -> OperationMode:
        """Return zone mode."""
        return self.mode

    def get_modes(self) -> list[OperationMode]:
        """Return zone modes."""
        modes = self.modes

        if len(modes) == 0 and self.mode is not None:
            modes = [self.mode]

        if OperationMode.STOP not in modes:
            modes += [OperationMode.STOP]

        return modes

    def get_name(self) -> str:
        """Return zone name."""
        return self.name

    def get_on(self) -> bool:
        """Return zone on/off."""
        return self.on

    def get_problems(self) -> bool:
        """Return zone problems."""
        return bool(self.errors)

    def get_sleep(self) -> SleepTimeout | None:
        """Return zone sleep time in minutes."""
        return self.sleep

    def get_speed(self) -> int | None:
        """Return zone speed."""
        return self.speed

    def get_speeds(self) -> list[int] | None:
        """Return zone speeds."""
        if len(self.speeds) > 0:
            return self.speeds
        return None

    def get_system_id(self) -> int | None:
        """Return system ID."""
        return self.system_id

    def get_system_zone_id(self) -> str:
        """Return combined system and zone ID."""
        return self.system_zone_id

    def get_temp(self) -> float:
        """Return zone temperature."""
        return round(self.temp, 2)

    def get_temp_max(self) -> float:
        """Return zone maximum temperature."""
        return self.fix_max_temp(self.temp_max)

    def get_temp_min(self) -> float:
        """Return zone minimum temperature."""
        return round(self.temp_min, 1)

    def get_temp_set(self) -> float | None:
        """Return zone set temperature."""
        if self.temp_set is not None:
            return round(self.temp_set, 1)
        return None

    def get_temp_step(self) -> float | None:
        """Return zone step temperature."""
        if self.temp_step is not None:
            return round(self.temp_step, 1)
        return None

    def get_temp_unit(self) -> TemperatureUnit:
        """Return zone temperature unit."""
        return self.temp_unit

    def is_stage_supported(self, stage: AirzoneStages) -> bool:
        """Check if Airzone Stage is supported."""
        cold_stages = self.get_cold_stages()
        if cold_stages is not None:
            if stage in cold_stages:
                return True
            if len(cold_stages) == 0:
                return True

        heat_stages = self.get_heat_stages()
        if heat_stages is not None:
            if stage in heat_stages:
                return True
            if len(heat_stages) == 0:
                return True

        return cold_stages is None and heat_stages is None

    def set_available(self, available: bool) -> None:
        """Set availability."""
        self.available = available

    def set_modes(self, modes: list[OperationMode]) -> None:
        """Set zone modes."""
        self.modes = modes

    def set_param(self, key: str, value: Any) -> None:
        """Update zone parameter by key and value."""
        if key == API_ANTI_FREEZE:
            self.anti_freeze = bool(value)
        elif key == API_COOL_SET_POINT:
            self.cool_temp_set = float(value)
        elif key == API_COLD_ANGLE:
            self.cold_angle = GrilleAngle(value)
        elif key == API_COLD_STAGE:
            self.cold_stage = AirzoneStages(value)
        elif key == API_ECO_ADAPT:
            self.eco_adapt = EcoAdapt(value)
        elif key == API_HEAT_ANGLE:
            self.heat_angle = GrilleAngle(value)
        elif key == API_HEAT_SET_POINT:
            self.heat_temp_set = float(value)
        elif key == API_HEAT_STAGE:
            self.heat_stage = AirzoneStages(value)
        elif key == API_MODE:
            self.mode = OperationMode(value)
        elif key == API_NAME:
            self.name = str(value)
        elif key == API_ON:
            self.on = bool(value)
        elif key == API_SET_POINT:
            self.temp_set = float(value)
        elif key == API_SLEEP:
            self.sleep = SleepTimeout(value)
        elif key == API_SPEED:
            self.speed = int(value)

    def validate_temp_set(self, temp_set: float, max_val: float | None) -> float | None:
        """Validate Zone temp set against its maximum value."""
        if max_val is not None and max_val != 0.0:
            if temp_set <= max_val:
                return temp_set
            return None
        return temp_set
